==> jils-0.4.0.tgz: package/lib/evaluate.js <==
var env = require('./env'),
	core = require('./core');

module.exports = { evaluate: evaluate };

function evaluate(_, global){
	if (core.stringp(_)){
		try {
			if (_.indexOf('.') > -1){
				var paths = _.split('.'),
					root = env.eget(global, paths[0]),
					value = root;
				for (var n = 1, l = paths.length; n < l; n++)
					value = value[paths[n]];
				value.__root = root;
				return value;
			}
			return env.eget(global, _);
		} catch(e) {
			return _;
		}
	} else if (core.hashp(_)){
		var keys = env.keys(_),
			values = env.values(_).map(function(v){ 
				return evaluate(v, global);
			});
		return env.mkhash(keys,values);
	} else if (!core.arrayp(_)){
		return _;
	} else if (_.length == 0){
		return [];
	} else {
		switch (_[0]){
			case 'quote':
				return _[1];
			case 'hash':
				var value = evaluate(_[1], global),
					hash = {};
				value.map(function(kov,i){
					if (i % 2) hash[lastk] = kov;
					else lastk = kov.replace(/^:/,'');
				});
				return hash;
			case 'list':
				return _.slice(1).map(function(v){
					return evaluate(v, global);
				})
			case 'if':
				var test = _[1],
					therefore = _[2],
					otherwise = _[3];
				var branch = evaluate(test, global) ?
					therefore : otherwise;
				return evaluate(branch, global);
			case 'def':
				var value = evaluate(_[2], global);
				return core.def(global, _[1], value);
			case 'fn':
				return function(){
					return evaluate(_[2],
						env.mkenv(_[1], arguments, global));
				}
			case 'do':
				var args = _.slice(1),
					result;
				for (var a in args)
					result = evaluate(args[a], global)
				return result;
			case 'eval':
				return evaluate(
					evaluate(_[1], global), 
				global);
			default:
				var proc = evaluate(_[0], global);
				if (proc.__macro){
					return evaluate(proc.apply(global, _.slice(1)), global); 
				}
				var exps = _.slice(1).map(function(exp){
					return evaluate(exp, global);
				});
				if (proc.__root){
					return proc.apply(proc.__root, exps);
				} return proc.apply && proc.apply(proc, exps);
		}
	}
}

==> jils-0.5.1.tgz: package/lib/evaluate.js <==
var env = require('./env'),
	core = require('./core');

module.exports = { evaluate: evaluate };

function evaluate(_, global){
	if (core.stringp(_)){
		try {
			if (_.indexOf('.') > -1){
				var paths = _.split('.'),
					root = env.eget(global, paths[0]),
					value = root;
				for (var n = 1, l = paths.length; n < l; n++)
					value = value[paths[n]];
				value.__root = root;
				return value;
			}
			return env.eget(global, _);
		} catch(e) {
			return _;
		}
	} else if (core.hashp(_)){
		var keys = env.keys(_),
			values = env.values(_).map(function(v){ 
				return evaluate(v, global);
			});
		return env.mkhash(keys,values);
	} else if (!core.arrayp(_)){
		return _;
	} else if (_.length == 0){
		return [];
	} else {
		switch (_[0]){
			case 'quote':
				return _[1];
			case 'hash':
				var value = evaluate(_[1], global),
					hash = {};
				value.map(function(kov,i){
					if (i % 2) hash[lastk] = kov;
					else lastk = kov.replace(/^:/,'');
				});
				return hash;
			case 'list':
				return _.slice(1).map(function(v){
					return evaluate(v, global);
				})
			case 'if':
				return evaluate(evaluate(_[1], global) ? _[2] : _[3], global);
			case 'def':
				var value = evaluate(_[2], global);
				return core.def(global, _[1], value);
			case 'loop':
				var cond = _[1];
					acc = [];
				while (evaluate(cond, global)){
					acc.push(evaluate(_[2], global));
				} return acc;
			case 'fn':
				var func = function(){
					return (function(keys, args){
						var _args = [], _keys = [];
						for (var k=0,a=0,l=args.length;a<l;a++,k++){
							if (keys[k] && keys[k].match(/^&/)){
								var nextk = keys.length - k - 1;
								_keys.push(keys[k].replace(/^&/,''));
								if (nextk == 0){ 
									_args.push(args.slice(a)); 
									break; 
								} else { 
									var slot = args.slice(a,args.length - nextk);
									_args.push(slot); a += slot.length - 1;
								}
							} 
							_args.push(args[a]);
							_keys.push(keys[k]);
						}
						return evaluate(_[2],
							env.mkenv(_keys, _args, global));
					})(_[1], core.slice(arguments));
				}; func.toString = function(){ return '[Function ('+_[1].join(' ')+')]' };
				return func;
			case 'do':
				var args = _.slice(1),
					result;
				for (var a in args)
					result = evaluate(args[a], global)
				return result;
			case 'eval':
				return evaluate(
					evaluate(_[1], global), 
				global);
			default:
				var proc = evaluate(_[0], global);
				if (proc.__macro){
					return evaluate(proc.apply(global, _.slice(1)), global); 
				}
				var exps = _.slice(1).map(function(exp){
					return evaluate(exp, global);
				});
				if (proc.__root){
					return proc.apply(proc.__root, exps);
				} return proc.apply && proc.apply(proc, exps);
		}
	}
}
